/// <reference types="node" />

interface Promise<T> {
	finally(callback: () => void): Promise<T>;
}
declare type UserRole = "all" | "approved" | "admin" | "owner";
declare enum DocumentStatuses {
	REVIEW = "review",
	REJECT = "reject",
	APPROVE = "approve",
	DRAFT = "draft"
}
declare enum EntityType {
	Folder = "Folder",
	Drawing = "Drawing",
	Flowchart = "Flowchart"
}
interface EntityAuthor {
	_id: string;
	name: string;
	avatar: string;
}
interface ProjectLinkMetadata {
	_id?: string;
	name?: string;
	link?: string;
}
export interface EntityData {
	_id: string;
	shortId: string;
	privateId?: string | null | false;
	type: EntityType;
	name: string;
	folder?: string;
	project?: string | ProjectLinkMetadata;
	team?: string;
	author?: EntityAuthor | string;
	modifiedAt?: Date | string;
	createdAt?: Date | string;
	updatedAt?: Date | string;
	openedAt?: Date | string;
	status?: EntityStatus;
	folderHierarchy?: EntityData[];
	depth?: number;
	children?: number;
	participants?: Participant[];
	password?: string;
	hasPassword?: boolean;
	isHidden?: boolean;
	archived?: boolean;
	threads?: {
		unread: number;
		unresolved: number;
	};
	cacheId?: string;
	shareType?: EntityShareTypes;
	collaborators?: Collaborator[];
	userRole?: UserRole;
}
interface EntityStatus {
	state: DocumentStatuses;
	notify: UserData[];
	lastChange: {
		at: Date;
		by: UserData;
	};
}
export interface UserData {
	_id: string;
	userType: string;
	name: string;
	avatar?: string;
	color?: string;
	workRole?: string;
	workTags?: string[];
	email?: string;
	unverifiedEmail?: string;
	plan?: string;
	newsletter?: boolean;
	newFeature?: string;
	lastTeam?: string;
	lastNewFeature?: string;
	hideNewFeatureNotifications?: boolean;
	muteAppNotifications?: boolean;
	isSuperAdmin?: boolean;
	isImpersonating?: boolean;
	pro?: boolean;
	oauth?: string;
	lastActive?: Date;
	tawkHash?: string;
	intercomHash?: string;
	intercomPayload?: string;
	createdAt?: string;
	settings?: string;
}
interface Participant extends Pick<UserData, "_id" | "name" | "userType" | "avatar"> {
	isOnline: boolean;
	role: UserRole | undefined;
}
declare enum EntityShareTypes {
	VISIBLE_TO_MEMBERS = 0,
	RESTRICTED_TO_SPECIFIC_PEOPLE = 1
}
interface Collaborator {
	_id: string;
	name: string;
	verified: boolean;
	role?: UserRole;
}
interface ProjectTreeItem {
	_id: string;
	name: string;
	folderCount: number;
}
export interface TeamTreeItem {
	_id: string;
	name: string;
	slug: string;
	avatar: string;
	projects: ProjectTreeItem[];
}
export interface ApiClientOptions {
	serverUrl?: string;
	apiToken: string;
}
export interface ListOptions {
	folder?: string;
	project?: string;
}
export interface MoveOptions {
	folder?: string;
	project?: string;
	entities: string[];
}
export interface WalkLocation {
	entity: EntityData;
}
export interface WalkContext {
	tree: WalkLocation[];
	entity: EntityData;
}
export interface UploadOptions {
	type: string;
	team?: string;
	folder?: string;
	project?: string;
	filename: string;
}
export interface CreateOptions {
	type: string;
	name?: string;
	project?: string;
	folder?: string;
}
export declare class ApiClient {
	private options;
	constructor(options: ApiClientOptions);
	serverUrl: string;
	private url;
	private baseRequest;
	private request;
	profile(): Promise<UserData>;
	get(id: string): Promise<EntityData>;
	hierarchy(id: string): Promise<EntityData>;
	download(id: string, type?: string): Promise<{
		stream: NodeJS.ReadableStream;
		entity: EntityData;
	}>;
	upload({ team, project, type, folder, filename }: UploadOptions, stream: NodeJS.ReadableStream | ArrayBuffer | Buffer): Promise<EntityData>;
	teams(): Promise<TeamTreeItem[]>;
	walkRecursively({ folder, project }: ListOptions, callback: (context: WalkContext) => Promise<void>): Promise<void>;
	list({ folder, project }: ListOptions): Promise<EntityData[]>;
	move({ folder, project, entities }: MoveOptions): Promise<{
		failure: string[];
		sucess: string[];
	}>;
	create(data: CreateOptions): Promise<unknown>;
	delete(id: string): Promise<unknown>;
	update(id: string, data: any): Promise<unknown>;
}

export {};
